// this file has been modified to work with the MSP430_IAR port...jcw
// you'll need to change this file to work with your target unless
// you're using similar clocking (6MHz XT2 MCLK) and using 115.2KB...
/*
	FreeRTOS V3.2.2 - Copyright (C) 2003 - 2005 Richard Barry.

	This file is part of the FreeRTOS distribution.

	FreeRTOS is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS, without being obliged to provide
	the source code for any proprietary components.  See the licensing section 
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license 
	and contact details.  Please ensure to read the configuration and relevant 
	port sections of the online documentation.
	***************************************************************************
*/


/* BASIC INTERRUPT DRIVEN SERIAL PORT DRIVER.   
 * 
 * This file only supports UART 1
 */

/* Standard includes. */
#include <stdlib.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

/* Demo application includes. */
#include "serial.h"

/* Constants required to setup the hardware. */
#define serTX_AND_RX			( ( unsigned portCHAR ) 0x03 )

// clock specific stuff - change as needed for your target...
#define XT2_FREQ     (6000000L)
#define SMCLK_FREQ   (XT2_FREQ/8)

// set host console baud rate here:
#define HOST_BAUD (115200UL)

#define UART_DIVISOR(baud_rate)    (SMCLK_FREQ / (baud_rate))
#define UART_REMAINDER(baud_rate)  (SMCLK_FREQ % (baud_rate))

#define HOST_REMAINDER UART_REMAINDER(HOST_BAUD)
#define HOST_DIVISOR   UART_DIVISOR(HOST_BAUD)

/* Misc. constants. */
#define serNO_BLOCK				( ( portTickType ) 0 )

/* Enable the UART Tx interrupt. */
#define vInterruptOn() IFG2 |= UTXIFG1

/* The queue used to hold received characters. */
static xQueueHandle xRxedChars; 

/* The queue used to hold characters waiting transmission. */
static xQueueHandle xCharsForTx; 

static volatile portSHORT sTHREEmpty;

/*-----------------------------------------------------------*/

xComPortHandle xSerialPortInitMinimal( unsigned portLONG ulWantedBaud, unsigned portBASE_TYPE uxQueueLength )
{
	unsigned portLONG ulBaudRateCount;
	
    unsigned long err;
    unsigned char modctl;
    unsigned int i;

	/* Initialise the hardware. */

	/* Generate the baud rate constants for the wanted baud rate. */
//	ulBaudRateCount = configCPU_CLOCK_HZ / ulWantedBaud;

	portENTER_CRITICAL();
	{
		/* Create the queues used by the com test task. */
		xRxedChars = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
		xCharsForTx = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
		/* Reset UART. */
		// UCTL1 |= SWRST;
         	U1CTL_bit.SWRST = 1;

		/* Set pin function. */
//		P4SEL |= serTX_AND_RX;
//		*P3SEL |= P3SEL_6 + P3SEL_7;
		P3SEL_bit.P3SEL_6 = 1;		// peripheral mode selected
		P3SEL_bit.P3SEL_7 = 1;		// peripheral mode selected	
//		*P3DIR |= P3DIR_6;
//		*P3DIR &= ~P3DIR_7;
		P3DIR_bit.P3DIR_6 = 1;		// 1 = output
		P3DIR_bit.P3DIR_7 = 0;		// 0 = input

		/* All other bits remain at zero for n, 8, 1 interrupt driven operation.
		LOOPBACK MODE!*/
//		U1CTL |= CHAR + LISTEN;
//		U1CTL |= CHAR;
//		U1TCTL |= SSEL1;
		U1CTL_bit.SYNC = 0;	// UART MODE
		U1CTL_bit.CHAR = 1;	// 8 bits
		U1CTL_bit.LISTEN = 1;
		U1CTL_bit.SPB = 0;	// one stop bit
		U1CTL_bit.PENA = 0;	// no parity 
		U1TCTL_bit.SSEL1 = 1;


		/* Setup baud rate low byte. */
		U1BR0 = ( unsigned portCHAR ) ( HOST_DIVISOR & ( unsigned portLONG ) 0xff );
//		U1BR0 = ( unsigned portCHAR ) ( ulBaudRateCount & ( unsigned portLONG ) 0xff );

		/* Setup baud rate high byte. */
//		ulBaudRateCount >>= 8UL;
		U1BR1 = ( unsigned portCHAR ) ( HOST_DIVISOR >> ( unsigned portLONG ) 8 );
//		U1BR1 = ( unsigned portCHAR ) ( ulBaudRateCount & ( unsigned portLONG ) 0xff );

    err = ulWantedBaud >> 1;
    modctl = 0;
    for (i = 0; i < 8; i++)
    {
	modctl >>= 1;
	err += HOST_REMAINDER;
	if (err >= ulWantedBaud)
	{
	    modctl |= 0x80;
	    err -= ulWantedBaud;
	}
    }
		/* Set. */
    //		UCTL1 &= ~SWRST;
    
                U1MCTL = modctl;
                U1CTL_bit.SWRST = 0;

                U1RCTL_bit.RXERR = 0;
		U1RCTL_bit.RXWAKE = 0;
		U1RCTL_bit.URXWIE = 0;
		U1RCTL_bit.URXEIE= 0;
		U1RCTL_bit.BRK = 0;
		U1RCTL_bit.OE = 0;
		U1RCTL_bit.PE = 0;
		U1RCTL_bit.FE = 0;
		    
		/* Nothing in the buffer yet. */
		sTHREEmpty = pdTRUE;
		
		/* Enable ports. */
		ME2 |= UTXE1 + URXE1;
 
		/* Enable interrupts. */
		IE2 |= URXIE1 + UTXIE1;
	}
	portEXIT_CRITICAL();
	
	/* Unlike other ports, this serial code does not allow for more than one
	com port.  We therefore don't return a pointer to a port structure and can
	instead just return NULL. */
	return NULL;
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialGetChar( xComPortHandle pxPort, signed portCHAR *pcRxedChar, portTickType xBlockTime )
{
	/* Get the next character from the buffer.  Return false if no characters
	are available, or arrive before xBlockTime expires. */
	if( xQueueReceive( xRxedChars, pcRxedChar, xBlockTime ) )
	{
		return pdTRUE;
	}
	else
	{
		return pdFALSE;
	}
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialPutChar( xComPortHandle pxPort, signed portCHAR cOutChar, portTickType xBlockTime )
{
signed portBASE_TYPE xReturn;

	/* Transmit a character. */

	portENTER_CRITICAL();
	{
		if( sTHREEmpty == pdTRUE )
		{
			/* If sTHREEmpty is true then the UART Tx ISR has indicated that 
			there are no characters queued to be transmitted - so we can
			write the character directly to the shift Tx register. */
			sTHREEmpty = pdFALSE;
			U1TXBUF = cOutChar;
			xReturn = pdPASS;
		}
		else
		{
			/* sTHREEmpty is false, so there are still characters waiting to be
			transmitted.  We have to queue this character so it gets 
			transmitted	in turn. */

			/* Return false if after the block time there is no room on the Tx 
			queue.  It is ok to block inside a critical section as each task
			maintains it's own critical section status. */
			xReturn = xQueueSend( xCharsForTx, &cOutChar, xBlockTime );

			/* Depending on queue sizing and task prioritisation:  While we 
			were blocked waiting to post on the queue interrupts were not 
			disabled.  It is possible that the serial ISR has emptied the 
			Tx queue, in which case we need to start the Tx off again
			writing directly to the Tx register. */
			if( ( sTHREEmpty == pdTRUE ) && ( xReturn == pdPASS ) )
			{
				/* Get back the character we just posted. */
				xQueueReceive( xCharsForTx, &cOutChar, serNO_BLOCK );
				sTHREEmpty = pdFALSE;
				U1TXBUF = cOutChar;
			}
		}
	}
	portEXIT_CRITICAL();

	return pdPASS;
}
/*-----------------------------------------------------------*/

#ifdef MSP_ROWLEY_RB_PORT

/* Serial interrupt service routines for the RB port. */

	/*
	 * UART RX interrupt service routine.
	 */
#pragma vector=UART1RX_VECTOR
__interrupt void vRxISR( void ) 
	{
	signed portCHAR cChar;
	
		/* Get the character from the UART and post it on the queue of Rxed 
		characters. */
		cChar = U1RXBUF;
	
		if( xQueueSendFromISR( xRxedChars, &cChar, pdFALSE ) )
		{
			/*If the post causes a task to wake force a context switch 
			as the woken task may have a higher priority than the task we have 
			interrupted. */
			taskYIELD();
		}
	}
	/*-----------------------------------------------------------*/
	
	/*
	 * UART Tx interrupt service routine.
	 */
#pragma vector=UART1TX_VECTOR	
__interrupt void vTxISR( void )
	{
	signed portCHAR cChar;
	portBASE_TYPE xTaskWoken;
	
		/* The previous character has been transmitted.  See if there are any
		further characters waiting transmission. */
	
		if( xQueueReceiveFromISR( xCharsForTx, &cChar, &xTaskWoken ) == pdTRUE )
		{
			/* There was another character queued - transmit it now. */
			U1TXBUF = cChar;
		}
		else
		{
			/* There were no other characters to transmit. */
			sTHREEmpty = pdTRUE;
		}
	}

#endif
/*-----------------------------------------------------------*/

#ifdef THAT_WHICH_IS_NOT_DEFINED

/* Serial port interrupts for the alternative port code. */

	void ISRCom1Rx( void )
	{
	signed portCHAR cChar;
	
		/* Get the character from the UART and post it on the queue of Rxed 
		characters. */
		cChar = U1RXBUF;
	
		if( xQueueSendFromISR( xRxedChars, &cChar, pdFALSE ) )
		{
			/*If the post causes a task to wake force a context switch 
			as the woken task may have a higher priority than the task we have 
			interrupted. */
			portEXIT_SWITCHING_ISR( pdTRUE );
		}
	}
	/*-----------------------------------------------------------*/
	
	void ISRCom1Tx( void )
	{
	signed portCHAR cChar;
	portBASE_TYPE xTaskWoken;
	
		/* The previous character has been transmitted.  See if there are any
		further characters waiting transmission. */
	
		if( xQueueReceiveFromISR( xCharsForTx, &cChar, &xTaskWoken ) == pdTRUE )
		{
			/* There was another character queued - transmit it now. */
			U1TXBUF = cChar;
		}
		else
		{
			/* There were no other characters to transmit. */
			sTHREEmpty = pdTRUE;
		}
	}

#endif
/*-----------------------------------------------------------*/
